@@ -9,6 +9,7 @@ from typing import (
     List,
     Iterable,
     Any,
+    Mapping,
 )
 
 import numpy as np
@@ -30,10 +31,16 @@ if TYPE_CHECKING:
 class ElasticConfig:
     n_dim: int  # dims  in elastic
     distance: str = 'cosine'  # similarity in elastic
-    hosts: str = 'http://localhost:9200'
+    hosts: Union[
+        str, List[Union[str, Mapping[str, Union[str, int]]]], None
+    ] = 'http://localhost:9200'
     index_name: Optional[str] = None
     es_config: Dict[str, Any] = field(default_factory=dict)
+    index_text: bool = False
+    tag_indices: List[str] = field(default_factory=list)
     batch_size: int = 64
+    ef_construction: Optional[int] = None
+    m: Optional[int] = None
 
 
 class BackendMixin(BaseBackendMixin):
@@ -82,19 +89,36 @@ class BackendMixin(BaseBackendMixin):
 
     def _build_schema_from_elastic_config(self, elastic_config):
         da_schema = {
-            "mappings": {
-                "dynamic": "true",
-                "_source": {"enabled": "true"},
-                "properties": {
-                    "embedding": {
-                        "type": "dense_vector",
-                        "dims": elastic_config.n_dim,
-                        "index": "true",
-                        "similarity": elastic_config.distance,
+            'mappings': {
+                'dynamic': 'true',
+                '_source': {'enabled': 'true'},
+                'properties': {
+                    'embedding': {
+                        'type': 'dense_vector',
+                        'dims': elastic_config.n_dim,
+                        'index': 'true',
+                        'similarity': elastic_config.distance,
                     },
+                    'text': {'type': 'text', 'index': elastic_config.index_text},
                 },
             }
         }
+        if elastic_config.tag_indices:
+            for index in elastic_config.tag_indices:
+                da_schema['mappings']['properties'][index] = {
+                    'type': 'text',
+                    'index': True,
+                }
+
+        if self._config.m or self._config.ef_construction:
+            index_options = {
+                'type': 'hnsw',
+                'm': self._config.m or 16,
+                'ef_construction': self._config.ef_construction or 100,
+            }
+            da_schema['mappings']['properties']['embedding'][
+                'index_options'
+            ] = index_options
         return da_schema
 
     def _build_client(self):
